## Overview

Wrapper of the TDVC library for 3DiVi BAF.

## Use cases

-   Check Liveness Reflection
-   Check Motion Control
-   Capture of faces

## Recommendations

Use a modern phone with a good camera having the definition of not less than 8 megapixels.
The capture must be made in a well-lighted room. A document must be located at the uniform background.

## Limitations

This component works on phones and desktops in the Chrome browser, Firefox, Safari, Mi Browser

## Installation

The component is supplied as a TGZ archive. Also, for the tdvc-onboarding library to work, the tdvc library is required, which is also supplied as an archive. Move the archives to your project's root folder and add the following lines to your package.json in the dependencies section:

```
"@tdvc/face-onboarding": "file:tdvc-face-onboarding-{version}.tgz"
```

This component contains JS, CSS files which require the mandatory import into your project.

Import the library and css to your project.

```javascript
import TDVOnboarding from '@tdvc/face-onboarding';
import '@tdvc/face-onboarding/dist/css/style.css';
```

For the component to work, the opencv.js script is required. After installing the package, move the "networks" folder from /node_modules/@tdvc/face-onboarding/ to the "public" folder of your project.

## Initialization

The component provides two modules: registration and authorization. To use the component, create an instance of the TDVOnboarding module you need. Each module accepts configuration object as parameters. Below is a description of configuration:

### Fields configurable via the configuration object

**baseUrl** (string) - URL address of API. If you pass the value "/", then requests will be sent to the host on which the web component is deployed. **Required**.

**integrationId** (string) - integration ID. **Required**.

**mountElement** (string) - ID of an element on the page where the component will be integrated. **Required**.

**applicantId** (string) - ID of applicant for registration or authorization.

> Please note that the configuration must only have one of two fields: applicantId or applicantFields.

**language** (string) - current interface language, if no value is specified, `“en”` will be used by default.

**locales** (Locales) - object with custom locales for changing texts. For ease of use, we recommend writing one file with locales and using it in all configurations of the component. If no value is specified, the default object will be used.

The accepted value is an object with the following content:

```javascript
const en = {
    Login: {
        Authorization: 'Authorization',
        Registration: 'Registration',
        LoginButton: 'Continue',
        InvalidEmail: 'Invalid email format',
        MaxLengthField: 'Maximum field length is 150 characters',
        IsRequired: 'This field is required',
        firstName: 'First name',
        lastName: 'Last name',
        phone: 'Phone',
        email: 'Email',
        referenceId: 'ReferenceId',
        WrongPhone: 'Wrong phone number',
        CheckApplicant: 'Applicant verification',
        LoadScripts: 'Load scripts',
        LoadModels: 'Loading models',
    },
    Register: {
        Header: 'Registration',
        RegisterDescription1: 'Registering as ',
        RegisterDescription2: 'Please complete the registration',
        RegisterButton: 'Register',
        BackToLoginButton: 'Back',
        DontMove: "Please don't move.",
        EstimateVideo: 'Checking the image quality',
        Update: 'Update',
        Processing: 'Processing in progress',
        LoadModels: 'Loading models',
        LoadVideoForm: 'Connecting the camera',
        LivenessDelay: 'Before recording starts',
    },
    Authorize: {
        Header: 'Authorization',
        AuthorizeDescription1: 'Authorizing as ',
        AuthorizeDescription2: 'Please complete the authorization',
        AuthorizeButton: 'Log in',
        BackToLoginButton: 'Back',
        DontMove: "Please don't move.",
        EstimateVideo: 'Checking the image quality',
        Update: 'Update',
        Processing: 'Processing in progress',
        LoadModels: 'Loading models',
        LoadVideoForm: 'Connecting the camera',
    },
    Errors: {
        MessageCode: 'Message code: ',
        AntispoofingValidationError: 'Liveness check failed.',
        RegistrationMatchingFailed: 'An applicant with such biometrics already exists.',
        AuthorizationMatchingFailed: 'No applicant with this biometric was found.',
        LowImageQuality: 'Poor image quality.',
        LivenessReflectionError: 'Liveness Reflection check failed.',
        NoFacesFound: 'No face found in the image.',
        SomeError: 'An error occurred, please try again later.',
        FacesDontBelongApplicant: 'Face matching check failed.',
        IDontSeeYou: 'You are not visible',
        MoveFaceOnCenter: 'Please position yourself so that your face is in the center of the circle on the screen',
        LookAtCamera: 'Please, turn your face to the camera',
        LittleFace: 'Move closer to the camera',
        BigFace: 'Move further away from the camera',
        SlowEnternet: 'Your internet connection is slow. Image quality assessment may take long',
        InCorrectCamera: 'The selected camera is not available or does not meet the minimum requirements.',
        NoCamera: 'No cameras available.',
        NoPermission:
            'Permission to access the camera is not obtained. For further work, allow access to the camera in your browser settings.',
        MoreFaces: 'Many faces in the frame.',
        SafariError: 'Unfortunately, your browser is temporarily not supported at the moment',
        BackToLoginButton: 'Back',
        ApplicantNotFound: 'Incorrect data entered',
        ApplicantAlreadyExist: 'This user already exists',
        Update: 'Update',
        ServerError: 'The server is temporarily unavailable.',
        ServerConfigError: 'Error on the server.',
        LoadScriptError: 'Loading script error.',
        TensorflowError: 'Error when running Tensorflow model',
        ApplicantBlocked: 'The applicant is blocked',
        MCCrossMatchError: 'Facial substitution detected.',
        LRCrossMatchError: 'Facial substitution detected.',
        BlackListError: 'The applicant is on the black list.',
        RiskError: 'During validation, the risk triggered.',
        "Face profile wasn't saved": "Face profile wasn't saved. Message code:",
        'Face profiles not found': 'Face profiles not found.',
        'Face authorization was failed': 'Face authorization was failed.',
        'A face was missing or disappeared from the frame when recording liveness reflection video':
            'A face was missing or disappeared from the frame when recording liveness reflection video.',
        'Invalid endeavor info': 'Invalid endeavor info',
        'Endeavor liveness reflection info obtain error': 'Error receiving liveness reflection information.',
        'Endeavor external link not equal to applicant': 'Applicant ID consistency error.',
        'Endeavor is not calculated': 'Error recording video liveness reflection.',
        'Endeavor liveness reflection confidence info is null': 'Error calculating liveness reflection.',
        'Endeavor liveness reflection confidence value is null': 'Error calculating liveness reflection.',
        'Failed to obtain template from liveness reflection reference image':
            'No face in the frame when recording video reflection.',
        'Failed to obtain template from motion control reference image':
            'No face in the frame when recording motion control video.',
        'Endeavor id is null when required': 'Endeavor id is null when required.',
        'No faces found on image': 'No faces found on image.',
        'Multiple faces found on image': 'Multiple faces found on image.',
        'Multiple faces or strong face movement spotted when recording liveness reflection video':
            'Multiple faces or strong face movement spotted when recording liveness reflection video.',
        'Error when perform cross matching': 'Error when perform cross matching.',
        'Error on the server': 'Error on the server.',
        'Liveness reflection video is not captured': 'Liveness reflection video is not captured.',
        'Liveness reflection video reference template is not captured':
            'Liveness reflection video reference template is not captured.',
        'Motion control video is not captured': 'Motion control video is not captured.',
        'Motion control video reference template is not captured':
            'Motion control video reference template is not captured.',
        NotSupportedMediaDevicesError: 'Media Devices API not supported.',
        AbortAccessToCameraError: 'Aborted access to camera.',
        DocumentIsNotFullyActiveError: 'HTML document is not fully active.',
        NotAllowedAccessToCameraError: 'Not allowed access to camera.',
        NotFoundCameraError: 'No camera was found.',
        NotReadableCameraError: 'The camera is unavailable because it is already in use by another process.',
        OverconstrainedCameraError: 'No camera satisfying the constraints of the system is found.',
        CameraSecurityError: 'The HTML document does not meet the minimum security requirements for camera use.',
        120004: 'FPS too low. Please try again.',
        120005: 'Error on the server. Please try again.',
        120006: 'Error on the server. Please try again.',
        120007: 'Error on the server. Please try again.',
        120008: 'Error on the server. Please try again.',
        120009: 'Please check the quality of your internet connection and try again.',
        120052: 'Error on the server. Please try again.',
        120053: 'Error on the server. Please try again.',
        120054: 'Error on the server. Please try again.',
        120055: 'Error while sending a message via DataChannel.',
        120057: 'Error in obtaining Motion Control pattern.',
        120060: 'Error on the server. Please try again.',
        120061: 'No face or multiple faces detected while taking reference image. Please try again.',
        180001: 'Invalid websocket message format.',
        180002: 'Failed to retrieve component settings.',
        180003: 'Requesting a video recording of an unsupported type.',
        180004: 'There was an error on the server while recording video.',
        180005: 'Video processing time has exceeded the limit.',
        180006: 'Error in operation of WebSocket connection.',
    },
    Success: {
        Register: 'Registration completed successfully',
        Authorize: 'Authorization completed successfully',
    },
    Liveness: {
        TurnLeft: 'Turn left',
        TurnRight: 'Turn right',
        TurnUp: 'Lift your chin up while continuing to look at the screen',
        LookAtCenter: 'Look into the camera',
        Closer: 'Move closer to the camera',
        Farther: 'Move further away from the camera',
        Normal: 'Return to the original position',
        TryFalled: 'Attempt failed',
        HandleResult: 'Processing the result',
        GoToNextStage: 'Moving on to the next stage',
    },
};
```

**callbacks** (TBLCallback) - object with callback hooks. Available next hooks:

-   **onValidate** ((data: any) => void) - the function that will be called after validation response received.

-   **onError** ((message: string, code: string) => void) - the function that will be called on error.

-   **onBack** (() => void) - a function that will be called after the back button is pressed.

-   **onUpdate** (() => void) - a function that will be called after the update button is pressed.

-   **onMotion** ((type: 'left' | 'right' | 'up' | 'closer' | 'farther' | 'center' | 'return', result: boolean | undefined) => void) - a function that will be called before and after each Motion control pattern. If the result variable is undefined, then the function was called at the beginning of the pattern, otherwise at the end.

-   **onMounted** (() => void) - a callback-function which will be called on the component mounted. This function doesn't return a value.

### Fields that are taken from the server by default and can be overwritten through the client

**livenessTransport** ('LivenessTransport.WEB_SOCKET' | 'LivenessTransport.WEB_RTC') - this setting allows you to select the type of communication between the client and the server (WebRTC or WebSocket) when passing checks on liveness.

**fingerprintWaitTime** (number) - this setting allows you to set the maximum waiting time for data collection to generate a fingerprint of device (Default is 2000 ms). Data that has not been received during this time will not be included in the fingerprint.

**faceModelSettings** (FaceDetectorSettings) - object with face mesh settings.

-   **modelEnabled** (boolean) - flag including a face mesh.

    > Please note that if you turn this option off, Motion Control will no longer be available. Also, Liveness Reflection video recording will start after a certain time. The default is true.

-   **timeToStartRecord** (number) - time in milliseconds after which Liveness Reflection recording will start if the face mesh is turned off.

**cameraSettings** (CameraSettings) - object with camera settings.

-   **showCameraSelect** (boolean) - flag allowing camera selection.

-   **cameraResolution** ('CameraResolutions.FULL_HD' | 'CameraResolutions.HD' | 'CameraResolutions.SD') - A parameter indicating the "ideal" video resolution. The "FULL_HD" key means 1920x1080 resolution, the "HD" key means 1280x720 resolution, and the "SD" key means 640x360 resolution. This setting does not indicate the exact resolution of the video stream, but only indicates the resolution that the browser will aim for. The default value is "HD".

-   **cameraId** (string) - ID of the camera to be used.

**motionControl** (MotionControlSettings) - object with Motion Control settings. The following fields are available:

-   **enabled** (boolean) - flag enabling Motion Control.

-   **attemptsCount** (number) - number of attempts a user can take to complete Motion Control.

-   **order** (number) - order of checks. **(WebSocket only)**

    > If the value matches the “order” value in the “livenessReflection” object, the Liveness Reflection will be checked first.

**livenessReflection** (LivenessReflectionSettings) - object with Liveness Reflection settings. The following fields are available:

-   **enabled** (boolean) - flag enabling Liveness Reflection. **(WebSocket only)**

-   **order** (number) - order of checks. **(WebSocket only)**

    > If the value matches the “order” value in the “motionControl” object, Liveness Reflection will be checked first.

**autoSubmit** (boolean) - flag that allows automatic sending of a check request after all checks have been passed.

**applicantFields** (ApplicantFields) - setting up applicant fields that should be displayed during registration or authorization. The values of these fields will be sent to the request to create an applicant. The primary field determines by which key the applicant will be searched in the database.

> Note that there must be only one primary field and it must be enabled. The accepted value is an object with the following content:

```javascript
applicantFields: {
    firstName: { enabled: false, primary: false },
    lastName: { enabled: false, primary: false },
    phone: { enabled: false, primary: false },
    email: { enabled: true, primary: true },
    referenceId: { enabled: false, primary: false },
};
```

**faceSettings** (FaceSettings) - object with recognition zone settings. This setting allows you to adjust the size of the displayed facial circumference. This setting also affects the face size at which video recording starts. The nested object sizes is responsible for the size of the standard circle. The circle for the **"closer"** pattern is determined by the parameter **closerPercent**, and for the pattern **"farther"** the parameter is **fartherPercent**. The parameters **closerPercent** and **fartherPercent** show how much the circle will be larger or smaller than the standard one. The **sizeAccuracy** parameter is responsible for the acceptable difference in the size of the displayed face circle and the BBox (measured in pixels). Example of an object with settings (the values ​​indicated in the example are the default values):

```javascript
faceSettings: {
    sizes: {
        mobile: {
            width: 1.2,
            height: 0.6,
        },
        pc: {
            sd: {
                width: 0.6,
                height: 0.9,
            },
            hd: {
                width: 0.6,
                height: 0.9,
            },
            fhd: {
                width: 0.4,
                height: 0.6,
            },
        },
    },
    closerPercent: 115,
    fartherPercent: 85,
    sizeAccuracy: 60,
}
```

**showMetrics** (boolean) - flag enabling display of WebRTC connection metrics: FPS and network **(WebRTC only)**.

**obtainReferenceFrame** (boolean) - flag enabling obtaining a reference frame for video.

**networksPath** (string) - a parameter indicating where FaceMesh models will be loaded from. The default value is **'/networks/'**.

### styling

To change styles, create your own css file, replace the necessary styles in it and import the file in your code. For convenience, the archive contains a css file with the described classes.

## Examples

Registration component initialization examples:

```javascript
import TDVOnboarding from '@tdvc/face-onboarding';
import '@tdvc/face-onboarding/dist/css/style.css';

const lib = new TDVOnboarding.Register({
    baseUrl: '/',
    integrationId: '295cb5c3-8d4f-4247-abc9-04de26548499',
    mountElement: 'app',
    callbacks: {
        onValidate(data) {
            console.log('onValidate callback', data);
        },
        onError(message, code) {
            console.log('onError callback', message, code);
        },
        onBack() {
            console.log('onBack callback');
        },
        onUpdate() {
            console.log('onUpdate callback');
        },
    },
});
```

```javascript
import TDVOnboarding, { ComponentSettingsFromClient } from '@tdvc/face-onboarding';
import { CameraResolutions } from '@tdvc/face-onboarding/dist/class/camera/const';
import { LivenessTransport } from '@tdvc/face-onboarding/dist/types';
import '@tdvc/face-onboarding/dist/css/style.css';

const config: ComponentSettingsFromClient = {
    baseUrl: '/',
    integrationId: '295cb5c3-8d4f-4247-abc9-04de26548499',
    mountElement: 'app',
    networksPath: '/networks/',
    language: 'en',
    locales,
    obtainReferenceFrame: true,
    livenessTransport: LivenessTransport.WEB_SOCKET,
    fingerprintWaitTime: 2000,
    showMetrics: false,
    autoSubmit: true,
    callbacks: {
        onValidate(data) {
            console.log('onValidate callback', data);
        },
        onError(message, code) {
            console.log('onError callback', message, code);
        },
        onBack() {
            console.log('onBack callback');
        },
        onMotion(type, currentAttemptNumber, result) {
            console.log('onMotion callback', type, currentAttemptNumber, result);
        },
        onUpdate() {
            console.log('onUpdate callback');
        },
        onMounted() {
            console.log('onMounted callback');
        },
    },
    cameraSettings: {
        showCameraSelect: true,
        cameraResolution: CameraResolutions.HD,
    },
    faceModelSettings: {
        modelEnabled: true,
        timeToStartRecord: 1000,
    },
    livenessReflection: {
        enabled: true,
        order: 0,
    },
    motionControl: {
        enabled: true,
        order: 1,
        attemptsCount: 3,
    },
    applicantFields: {
        firstName: { enabled: false, primary: false },
        lastName: { enabled: false, primary: false },
        phone: { enabled: false, primary: false },
        email: { enabled: true, primary: true },
        referenceId: { enabled: false, primary: false },
    },
    faceSettings: {
        sizes: {
            mobile: { width: 1.2, height: 0.6 },
            pc: {
                fhd: { width: 0.6, height: 0.9 },
                hd: { width: 0.6, height: 0.9 },
                sd: { width: 0.6, height: 0.9 },
            },
        },
        closerPercent: 115,
        fartherPercent: 85,
        sizeAccuracy: 40,
    },
}

const lib = new TDVOnboarding.Register(config);
```

## Version table

Version compatibility table:

<table>
    <tbody>
        <tr>
            <td class="center-aligned" width="400">
                <b>ServerBAF</b>
            </td>
            <td class="center-aligned" width="400">
                <b>@tdvc/face-onboarding</b>
            </td>
        </tr>
           <tr>
            <td width="400">1.8.*</td>
            <td width="400">1.8.*</td>
        </tr>
         <tr>
            <td width="400">1.7.0</td>
            <td width="400">1.7.0</td>
        </tr>
        <tr>
            <td width="400">1.5.0-1.7.0</td>
            <td width="400">1.6.0</td>
        </tr>
         <tr>
            <td width="400">1.5.0</td>
            <td width="400">1.5.0</td>
        </tr>
        <tr>
            <td width="400">1.3.0-1.4.0</td>
            <td width="400">1.4.0-1.4.1</td>
        </tr>
        <tr>
            <td width="400">1.3.0-1.3.1</td>
            <td width="400">1.3.1</td>
        </tr>
        <tr>
            <td width="400">1.3.0-1.3.1</td>
            <td width="400">1.3.1</td>
        </tr>
         <tr>
            <td width="400">1.2.0</td>
            <td width="400">1.2.0-1.3.0</td>
        </tr>
        <tr>
            <td width="400">1.1.0</td>
            <td width="400">1.0.0-1.1.2</td>
        </tr>
        <tr>
            <td width="400">1.0.0</td>
            <td width="400">1.0.0</td>
        </tr>
    </tbody>
</table>

## Version history

-   **1.8.0**

    -   HTTP liveness transport replaced by WebSocket
    -   Added ability to customize the order of checks and disable Motion Control and Liveness Reflection **(WebSocket only)**
    -   Fixed a bug that causes the validation request to be executed before the server processes Liveness Reflection and Motion Control data when “autoSubmit” is enabled.
    -   Fixed a bug where face settings did not affect Motion Control checks
    -   Fixed a bug in the web component that resulted in multiple requests to access the camera in telegram.
    -   Fixed a bug that caused the component to hang on a mobile device if access to the camera was allowed in the web session, but access to the camera was denied in the application permissions.

-   **1.7.2**

    -   Added ability to overwrite component settings received from the server through client-side configs
    -   Added error display if a request to retrieve component settings fails
    -   Fixed a bug that caused “Server Error” to be displayed instead of validation error
    -   Fixed a bug that caused the component to slow down when the “Update” button was clicked multiple times

-   **1.7.1**

    -   Fixed a bug that caused the component display to break after clicking on the “Update” button when receiving error 120061

-   **1.7.0**

    -   Added configuration of business logic components on the server
    -   The configuration has been updated:
        -   The **_fingerprintWaitTime_** field has been added.
        -   **_lrsToken_** replaced on **_videoRecorderToken_**.
        -   **_lrsDecryptKey_** replaced on **_videoRecorderDecryptionKey_**
    -   Fixed a bug that caused the Liveness check don't start when setting the value to 0 for the **_timeToStartRecord_** variable.
    -   Fixed a bug that caused a processing message to be displayed and it was not possible to continue working with the component.
    -   Fixed a bug that caused the processing window to be displayed indefinitely after passing Motion Control
    -   Updated TDVC library to 1.5.3 version.
    -   Update README.md

-   **1.6.3**

    -   The configuration has been updated: the **_livenessTransport_** field has been added to the **_blConfig_** object. This setting allows you to select the type of communication between the client and the server (WebRTC or HTTP) when passing checks on liveness.
    -   Update README.md
    -   Updated TDVC library to 1.5.2 version.

-   **1.6.2**

    -   The configuration has been updated: the **_faceModelSettings_** field has been added to the **_blConfig_** object. In these settings, you can enable the use of a face grid, as well as adjust the time before recording starts. Read more in the configuration description in this file. Checking the update configuration.
    -   Update README.md
    -   Updated TDVC library to 1.5.1 version.

-   **1.6.1**

    -   Fix README.md
    -   Added missing recognition models to the "networks" folder.

-   **1.6.0**

    -   Updated TDVC library to 1.5.0 version.
    -   Deleted **_platformToken_** from **_serverConfig_**.
    -   Updated using **_TDVC.API.getApplicantByFilter_** methods. Add **_page_** and **_pageSize_** parameters.
    -   Updated using **_TDVC.API.getLRSOffer_** methods. Delete **_any_** type annotation.
    -   Added Tensowflow models in networks directory. Fix tensorflow initial and destroy.
    -   The configuration has been updated: the **_cameraSettings_** field has been added to the **_blConfig_** object. In these settings you can enable camera selection in the web component or pass cameraId. You can also specify the required video stream resolution. Update config validation.
    -   Fixed error with camera selector after Liveness Reflection.
    -   Added risk processing during registration and authorization.
    -   Added package version printing.
    -   Added text for LRS errors.
    -   Added **_onMounted_** callback. It's a callback-function which will be called on the component mounted
    -   Added **_Version compatibility table_** in README.md.
    -   Added **_networksPath_** parameter in **_serverConfig_**.

-   **1.5.0**

    -   Add size accuracy in faceSettings config
    -   Update custom-style.css file
    -   Add sending Motion Control result on server

-   **1.4.1**

    -   Update Motion Control config
    -   Add motion controll attempts count function
    -   Update config validation
    -   Update locales
    -   Update onMotion callback
    -   Fix camera changing
    -   Fix Keypoints canvas view
    -   Add face sizes settings

-   **1.4.0**

    -   Add dynamic face circle
    -   Add new callback - onBack
    -   Add new backendUrl pattern - if you pass the value "/", then requests will be sent to the host on which the web component is deployed
    -   Add OpenCV script in package
    -   Add enabled WebRTC metrics: FPS and network bitrate
    -   Fix bug with form validation
    -   Add auto submit feature
    -   Add onMotion callback
    -   Add onUpdate callback
    -   Fixed logic for checking applicant status
    -   Fix bug with WebRTC stop connection
    -   Update TDVC lib to 1.4.0 version
    -   Add new errors checking
    -   Update video angles

-   **1.3.1**

    -   Added processing of Liveness Reflection check results
    -   Add new message for face capturing
    -   Performance improvement
    -   Update TDVC lib to 1.3.1

-   **1.3.0**

    -   Add applicantID in blConfig
    -   Add locales for liveness reflection calculating
    -   Fix server error message view
    -   Update TDVC lib to 1.3.0

-   **1.2.0**

    -   Add active liveness with head turns
    -   Optimize works with backend
    -   Change applicationFields and iceServers configs
    -   Add new styles and locales
    -   Fix bugs

-   **1.1.2**

    -   Update FaceMesh lib.
    -   Fix bugs with Safari browser

-   **1.1.1**

    -   Update Liveness Reflection algorithm.
    -   Fix bugs

-   **1.1.0**

    -   First release
