import { CameraResolutions } from '../class/camera/const';
import { ILocalizator, Locales } from '../class/localizator';
export type TServerConfig = {
    baseUrl: string;
    videoRecorderToken: string;
    videoRecorderDecryptionKey: string;
    iceServers: IceCandidates[];
    networksPath: string;
};
export type TInterfaceConfig = {
    mountElement: string;
    showMetrics: boolean;
    localizator: ILocalizator;
    faceSettings: FaceSettings;
};
export type TFaceModelConfig = {
    modelEnabled: boolean;
    timeToStartRecord: number;
};
export type TBLCallback = {
    onValidate?: (data: any) => void;
    onError?: (message: string, code: string) => void;
    onBack?: () => void;
    onMotion?: (type: 'left' | 'right' | 'up' | 'closer' | 'farther' | 'center' | 'return', currentAttemptNumber: number, result?: boolean) => void;
    onUpdate?: () => void;
    onMounted?: () => void;
};
export type TBLConfig = {
    cameraSettings: CameraSettings;
    faceModelSettings: TFaceModelConfig;
    livenessTransport: LivenessTransport;
    applicantId?: string;
    applicantFields?: ApplicantFields;
    motionControl: MotionControlSettings;
    livenessReflection: LivenessReflectionSettings;
    autoSubmit: boolean;
    fingerprintWaitTime: number;
    callbacks?: TBLCallback;
    baseUrl: string;
    obtainReferenceFrame: boolean;
};
export type ComponentConfiguration = {
    baseUrl: string;
    integrationId: string;
    mountElement: string;
    applicantId?: string;
    callbacks?: TBLCallback;
    networksPath: string;
    videoRecorderToken: string;
    videoRecorderDecryptionKey: string;
    showMetrics: boolean;
    livenessTransport: LivenessTransport;
    obtainReferenceFrame: boolean;
    autoSubmit: boolean;
    fingerprintWaitTime: number;
    iceCandidates: IceCandidates[];
    faceModelSettings: FaceDetectorSettings;
    cameraSettings: CameraSettings;
    motionControl: MotionControlSettings;
    livenessReflection: LivenessReflectionSettings;
    applicantFields: ApplicantFields;
    faceSettings: FaceSettings;
};
export type ComponentSettingsFromClient = {
    baseUrl: string;
    integrationId: string;
    mountElement: string;
    networksPath?: string;
    applicantId?: string;
    language?: string;
    locales?: Locales;
    callbacks?: TBLCallback;
    cameraSettings?: CameraSettings;
    livenessTransport?: LivenessTransport;
    fingerprintWaitTime?: number;
    showMetrics?: boolean;
    autoSubmit?: boolean;
    faceModelSettings?: FaceDetectorSettings;
    livenessReflection?: LivenessReflectionSettings;
    motionControl?: MotionControlSettings;
    applicantFields?: ApplicantFields;
    faceSettings?: FaceSettings;
    obtainReferenceFrame?: boolean;
};
export type ComponentMetaFromServer = {
    integrationId: string;
    domain: string;
    description: string;
    active: true;
    settings: ComponentSettingsFromServer;
};
export type ComponentSettingsFromServer = {
    networksPath: string;
    videoRecorderToken: string;
    videoRecorderDecryptionKey: string;
    showMetrics: boolean;
    livenessTransport: LivenessTransport;
    obtainReferenceFrame: boolean;
    autoSubmit: boolean;
    fingerprintWaitTime: number;
    iceCandidates: IceCandidates[];
    faceModelSettings: FaceDetectorSettings;
    cameraSettings: CameraSettings;
    motionControl: MotionControlSettings;
    livenessReflection: LivenessReflectionSettings;
    applicantFields: ApplicantFields;
    faceSettings: FaceSettings;
};
export declare enum LivenessTransport {
    WEB_RTC = "WebRTC",
    WEB_SOCKET = "WebSocket"
}
export type IceCandidates = {
    iceCandidateType: IceCandidateType;
    iceCandidateTransport: IceCandidateTransport;
    ipAddress: string;
    port: number;
};
export declare enum IceCandidateType {
    STUN = "stun",
    TURN = "turn"
}
export declare enum IceCandidateTransport {
    TCP = "tcp",
    UDP = "udp"
}
export type FaceDetectorSettings = {
    modelEnabled: boolean;
    timeToStartRecord: number;
};
export type CameraSettings = {
    showCameraSelect: boolean;
    cameraResolution: CameraResolutions;
    cameraId?: string;
};
export type LivenessReflectionSettings = EnabledModule;
export type MotionControlSettings = EnabledModule & {
    attemptsCount: number;
};
export type EnabledModule = {
    enabled: boolean;
    order: number;
};
export type ApplicantFields = {
    firstName: ApplicantField;
    lastName: ApplicantField;
    phone: ApplicantField;
    email: ApplicantField;
    referenceId: ApplicantField;
};
export type ApplicantField = {
    enabled: boolean;
    primary: boolean;
};
export type FaceSettings = {
    sizes: {
        mobile: FaceSize;
        pc: {
            fhd: FaceSize;
            hd: FaceSize;
            sd: FaceSize;
        };
    };
    closerPercent: number;
    fartherPercent: number;
    sizeAccuracy: number;
};
export type FaceSize = {
    width: number;
    height: number;
};
